/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.editors.panels;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableItem;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.dialogs.EditAttrDialog;

public class AttributesPanel extends Composite {
    private static final String columnNames[] = new String[] { "Name", "Value" };

    private static final String TITLE_REMOVE_ATTR = "Remove attribute";
    private static final String TITLE_ADD_ATTR = "Add attribute";
    private static final String BUT_REMOVE = "Remove...";
    private static final String BUT_ADD = "Add...";

    private Map<String, Object> attributes;
    private Map<String, Object> innerAttr= new HashMap<String, Object>();
    private TableViewer attribTable;

    private Button addAttrButton = null;
    private Button removeAttrButton = null;

    private Listener addAttrListener = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown && event.keyCode == SWT.INSERT)
                    || (event.type == SWT.Selection)) {
                EditAttrDialog addAttrDialog = new EditAttrDialog(getShell(),"", "",innerAttr);
                if (addAttrDialog.open() == EditAttrDialog.OK) {
                    innerAttr.put(addAttrDialog.getKey(), addAttrDialog.getValue());
                    attribTable.setInput(innerAttr);
                    attribTable.getControl().setFocus();
                    AttributesPanel.this.notifyListeners(SWT.Modify,
                            new Event());
                }
            }
        }
    };

    private Listener removeAttrLink = new Listener() {
        @Override
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown && event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {

                TableItem[] items = attribTable.getTable().getSelection();
                String mes = items.length <= 1 ? "Are you sure you want to remove selected attribute?"
                        : "Are you sure you want to remove selected attributes?";

                if ((items.length > 0)
                        && (MessageDialog.openQuestion(getShell(),
                                "Attribute Editor", mes) == true)) {

                    for (int i = 0; i < items.length; i++) {
                        Object attrName = ((Object [])items[i].getData())[0];
                        innerAttr.remove(attrName);
                        attribTable.refresh();
                        attribTable.getControl().setFocus();
                        AttributesPanel.this.notifyListeners(SWT.Modify,
                                new Event());
                    }
                }
            }
        }
    };

    public AttributesPanel(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(2, false));
        attribTable = new TableViewer(this);
        attribTable.setSorter(new ViewerSorter());
        attribTable.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        // Create columns
        for (String colName : columnNames) {
            TableViewerColumn column = new TableViewerColumn(attribTable, SWT.H_SCROLL | SWT.V_SCROLL);
            column.getColumn().setText(colName);
            column.getColumn().setWidth(100);
            column.getColumn().setResizable(true);
            column.getColumn().setMoveable(true);
        }
        attribTable.setColumnProperties(columnNames);
        attribTable.getTable().setHeaderVisible(true);
        attribTable.getTable().setLinesVisible(true);

        // Editors
        CellEditor[] editors = new CellEditor[columnNames.length];
        editors[0] = new TextCellEditor(attribTable.getTable());
        editors[1] = new TextCellEditor(attribTable.getTable());
        attribTable.setCellEditors(editors);
//        attribTable.setCellModifier(new ICellModifier() {
//            @Override
//            public void modify(Object element, String property, Object value) {
//                TableItem ti = (TableItem) element;
//                if(property.equals(columnNames[0])) {
//                    innerAttr.remove(ti.getText(0));
//                    innerAttr.put((String)value, ti.getText(1));
//                } else
//                if(property.equals(columnNames[1])) {
//                    innerAttr.put(ti.getText(0), (String) value);
//                }
//                AttributesPanel.this.notifyListeners(SWT.Modify, new Event());
////                attribTable.update(element, new String[] {property});
////                attribTable.refresh();
//            }
//
//            @Override
//            public Object getValue(Object element, String property) {
//                int index = Arrays.asList(columnNames).indexOf(property);
//                return ((Object[]) element)[index];
//            }
//
//            @Override
//            public boolean canModify(Object element, String property) {
//                return false;
//            }
//        });
        attribTable.addDoubleClickListener(new IDoubleClickListener() {
            @Override
            public void doubleClick(DoubleClickEvent event) {
                Object[] selection = (Object[]) ((IStructuredSelection) event.getSelection()).getFirstElement();
                EditAttrDialog addAttrDialog = new EditAttrDialog(getShell(),(String)selection[0], (String)selection[1],innerAttr, false);
                if (addAttrDialog.open() == EditAttrDialog.OK) {
                    innerAttr.remove(selection[0]);
                    innerAttr.put(addAttrDialog.getKey(), addAttrDialog.getValue());
                    attribTable.setInput(innerAttr);
                    attribTable.getControl().setFocus();
                    AttributesPanel.this.notifyListeners(SWT.Modify, new Event());
                    attribTable.refresh();
                }
            }
        });

        attribTable.setLabelProvider(new ITableLabelProvider() {
            @Override
            public void removeListener(ILabelProviderListener arg0) {
            }

            @Override
            public void addListener(ILabelProviderListener arg0) {
            }

            @Override
            public boolean isLabelProperty(Object arg0, String arg1) {
                return true;
            }

            @Override
            public void dispose() {
            }

            @Override
            public String getColumnText(Object element, int columnIndex) {
                Object[] row = (Object[]) element;
                return row[columnIndex].toString();
            }

            @Override
            public Image getColumnImage(Object element, int columnIndex) {
                return null;
            }
        });
        attribTable.setContentProvider(new IStructuredContentProvider() {
            @Override
            public Object[] getElements(Object input) {
                if (input instanceof Map<?, ?>) {
                    Map<?, ?> attrs = (Map<?, ?>) input;
                    ArrayList<Object[]> data = new ArrayList<Object[]>();
                    for (Object key : attrs.keySet()) {
                        data.add(new Object[] { key, attrs.get(key) });
                    }
                    return data.toArray(new Object[0]);
                }
                return null;
            }

            @Override
            public void dispose() {
            }

            @Override
            public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
            }
        });

        attribTable.addSelectionChangedListener(new ISelectionChangedListener() {
                    @Override
                    public void selectionChanged(SelectionChangedEvent event) {
                        updateUI();
                    }
                });
        
        // Create buttons

        Composite attrsButtons = new Composite(this, SWT.NONE);
        attrsButtons.setLayout(new GridLayout());

        addAttrButton = new Button(attrsButtons, SWT.NONE);
        addAttrButton.setText(BUT_ADD);
        addAttrButton.setToolTipText(TITLE_ADD_ATTR);
        addAttrButton.addListener(SWT.Selection, addAttrListener);
        addAttrButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        removeAttrButton = new Button(attrsButtons, SWT.NONE);
        removeAttrButton.setText(BUT_REMOVE);
        removeAttrButton.setToolTipText(TITLE_REMOVE_ATTR);
        removeAttrButton.addListener(SWT.Selection, removeAttrLink);
        removeAttrButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        // removeAttrButton.addListener(SWT.KeyDown, buttonLink);

        updateUI();
    }

    protected void updateUI() {
        // TODO: massive deletion of attributes
        if (attribTable.getTable().getSelection().length == 1) {
            removeAttrButton.setEnabled(true);
        } else {
            removeAttrButton.setEnabled(false);
        }
    }

    public void setAttributes(TreeNode req) {
        innerAttr = new HashMap<String, Object>();
        for(String key : req.getAttributeKeys()) {
            if(key.startsWith("_")) {
                continue;
            }
            innerAttr.put(key, req.getAttributeValue(key).toString());
        }
        this.attributes = Collections.unmodifiableMap(innerAttr);
        attribTable.setInput(innerAttr);
    }

    public Map<String, Object> getAttributes() {
        return Collections.unmodifiableMap(innerAttr);
    }

    public boolean isChanged() {
        if (!attributes.equals(innerAttr))
            return true;
        return false;
    }
}
