/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.editors.panels;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Text;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectReqDialog;

public class SelectNodePanel extends AbstractPanel {
    public static final String ROOT_STRING = "/";

    private static final String BTN_REQ = "Browse...";

    // Widgets
    private Text rootText = null;
    private Button rootBut;

    private SelectNodeDialog selecNodeDialog;

    private final TreeDB db;
    private TreeNode importRoot;
    private TreeNode curRoot;
    private boolean readOnly = false;

    private boolean canSelectRoot = false;

    public void setNode(TreeNode req) {
        importRoot = req;
        curRoot = req;
        if (rootText != null && !rootText.isDisposed())
            rootText.setText(req.getId());
    }
    public SelectNodePanel(Composite parent, int style, TreeDB db) {
        this(parent, style, db, null);
    }
    public SelectNodePanel(Composite parent, int style, TreeDB db,
            SelectNodeDialog selecNodeDialog) {
        super(parent, style);
        this.selecNodeDialog = selecNodeDialog;
        this.setLayout(new GridLayout(2, false));
        this.db = db;
        this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        rootText = new Text(this, SWT.BORDER);
        GridData gdata = new GridData(GridData.FILL_HORIZONTAL);
        gdata.grabExcessHorizontalSpace = true;
        rootText.setLayoutData(gdata);
        rootText.setEditable(false);
        rootText.setText(ROOT_STRING);
        rootBut = new Button(this, SWT.NONE);
        rootBut.setText(BTN_REQ);
        SelectionListener rootList = new SelectionListener() {
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
            }

            @Override
            public void widgetSelected(SelectionEvent e) {
                if (SelectNodePanel.this.selecNodeDialog == null) {
                    SelectNodePanel.this.selecNodeDialog = new SelectReqDialog(
                            getShell(), SelectNodePanel.this.db);
                }

                SelectNodePanel.this.selecNodeDialog
                        .setCanSelectRoot(canSelectRoot);

                if (SelectNodePanel.this.selecNodeDialog.open() == SelectNodeDialog.OK) {
                    TreeNode selected = SelectNodePanel.this.selecNodeDialog
                            .getResult();
                    if (selected != null) {
                        rootText.setText(selected.getId());
                        curRoot = selected;
                        SelectNodePanel.this.notifyListeners(SWT.Modify,
                                new Event());
                    }
                }
            }
        };
        rootBut.addSelectionListener(rootList);
        initModifyListener();
    }

    @Override
    protected void initModifyListener() {
        rootText.addListener(SWT.Modify, super.modifyListener);
    }

    @Override
    public boolean isChanged() {
        return !importRoot.equals(curRoot);
    }

    public TreeNode getSelectedNode() {
        return curRoot;
    }

    public void setCanSelectRoot(boolean canSelectRoot) {
        this.canSelectRoot = canSelectRoot;
    }

    public boolean isReadOnly() {
        return readOnly;
    }

    public void setReadOnly(boolean readOnly) {
        rootText.setEditable(!readOnly);
        rootBut.setEnabled(!readOnly);
        this.readOnly = readOnly;
    }

    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        rootText.setEnabled(enabled);
        rootBut.setEnabled(enabled);
    }
}
