package com.unitesk.requality.eclipse.ui.cnf;

import java.text.Collator;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;

public class RequalitySorter extends ViewerSorter {

    public RequalitySorter() {
    }

    public RequalitySorter(Collator collator) {
        super(collator);
    }

    @Override
    public int compare(Viewer viewer, Object e1, Object e2) {
        if (!(e1 instanceof TreeNode)) {
            return super.compare(viewer, e1, e2);
        }
        if (!(e2 instanceof TreeNode)) {
            return super.compare(viewer, e1, e2);
        }
        TreeNode n1 = (TreeNode) e1;
        TreeNode n2 = (TreeNode) e2;
        if (!n1.getTreeDB().equals(n2.getTreeDB())) {
            return super.compare(viewer, e1, e2);
        }

        if (n1.getType().compareTo(n2.getType()) != 0) {
            return n1.getType().compareTo(n2.getType());
        }
        if (n1 instanceof Location && n2 instanceof Location && n1.getParent().getQualifiedId().equals(n2.getParent().getQualifiedId()) && n1.getParent() instanceof Document) {
            String id1 = n1.getId(), id2 = n2.getId();
            if (id1.equals(id2)) {
                return 0;
            }
            List<UUID> locs = ((Document) n1.getParent()).getLocations();
            int i1 = -1, i2 = -1;
            for (int i = 0; i < locs.size(); i++) {
                String id = locs.get(i).toString();
                if (id.equals(id1)) {
                    i1 = i;
                }
                if (id.equals(id2)) {
                    i2 = i;
                }
                if (i1 >= 0 && i2 >= 0) {
                    break;
                }
            }
            return i1 - i2;
        }
        int result;
        if((result = getCompareInteger(n1).compareTo(getCompareInteger(n2)))==0) {
            return n1.getId().compareToIgnoreCase(n2.getId());
        }
        return result;
    }

    private Integer getCompareInteger(TreeNode n) {
        Integer val = 0;
        try {
            String attrName = getSortAttribute(TreesTracker.getTracker()
                    .getProject(n.getTreeDB()));
            val = Integer
                    .valueOf((String) n.getAttributeValue(attrName,
                            getDefaultAttributeValue(attrName, n)).toString());
        } catch (Exception e) {
            // ignore wrong attribute format
        }
        return val;
    }

    private String getSortAttribute(IProject project) {
        return TreeDB.ORDER_ATTRIBUTE;
    }

    private Integer getDefaultAttributeValue(String attrName, TreeNode node) {
        return 0;
    }
}
