package com.unitesk.requality.eclipse.ui.editors;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.NodeMapper;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.handlers.location.CommonRemoveLocation;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tools.RequalityCLI;

public abstract class JSEditor extends EditorPart implements ISelectionProvider {

    public static final int EDIT_MODE_REQ_RO = 0x01;
    public static final int EDIT_MODE_TP_RO = 0x02;
    private Browser browser;
    protected File html;
    private TreeNode node;
    private TreeNode currentSelection;
    private TreeDB db;
    private UniEditorOutlinePage uniEditorOutlinePage;
    private int editMode = 0;

    private ITreeStateListener treeStateListener = new ITreeStateListener() {
        
        @Override
        public void open(TreeDB db) {
            // TODO Auto-generated method stub
            
        }
        
        @Override
        public void close(TreeDB db) {
            if(!JSEditor.this.db.equals(db)) {
                return;
            }
            closeEditor();
        }

        @Override
        public void delete(IProject proj) {
            IProject project = TreesTracker.getTracker().getProject(db);
            if (!project .equals(proj)) {
                return;
            }
            closeEditor();
        }

    };

    private ISelectionChangedListener outlineSelectionChangeListener = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            IStructuredSelection selection = (IStructuredSelection) event
                    .getSelection();
            if (selection.isEmpty()) {
                return;
            }
            setUniEditorFocus(((TreeNode) selection.getFirstElement())
                    .getUUId());
            currentSelection = (TreeNode) selection.getFirstElement();
            // synchronization with Requality Explorer
            IViewPart view = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage()
                    .findView(RequalityCNF.ID);
            if (view instanceof CommonNavigator) {
                CommonNavigator navigator = (CommonNavigator) view;
                if (navigator.isLinkingEnabled()) {
                    navigator.setLinkingEnabled(true);
                }
            }
        }
    };

    public TreeNode getCurrentSelection() {
        if(currentSelection==null) {
            currentSelection = targetNode;
        }
        return currentSelection;
    }

    public void setSelection(TreeNode node) {
        setUniEditorFocus(node.getUUId());
        currentSelection = node;
        if (uniEditorOutlinePage != null
                && !uniEditorOutlinePage.getControl().isDisposed()) {
            uniEditorOutlinePage.setSelection(new StructuredSelection(node));
        }
    }


    private Map<UUID, NodeDesc> changedNodes;
    private Map<UUID, Integer> changedNodesCount;
    ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            refresh(node.getParent());
            refresh(oldParent);
        }

        @Override
        public void deleted(NodeDesc nodeDesc) {
            if(!db.getStorage().canLoad(JSEditor.this.node.getUUId())) { // current node was deleted
                closeEditor();
                return;
            }
            if(db.getStorage().canLoad(nodeDesc.getParentUUId())) {
                refresh(db.getNode(nodeDesc.getParentUUId()));
            }
        }

        @Override
        public void changed(TreeNode node) {
            refresh(node);
        }

        @Override
        public void created(TreeNode node) {
            refresh(node.getParent());
        }
    };
    private TreeNode targetNode;

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        changedNodes = new HashMap<UUID, NodeDesc>();
        changedNodesCount = new HashMap<UUID, Integer>();
        try {
            node = ((NodeEditorInput) input).getStorage().getNode();
            db = node.getTreeDB();
            String partName;
            if (node instanceof Requirement) {
                partName = ((Requirement) node).getName();
            } else {
                partName = node.toString();
            }
            Object targetNode = ((NodeEditorInput)input).getData();
            if(targetNode instanceof TreeNode) {
                this.targetNode = (TreeNode) targetNode;
                partName += " ("+targetNode.toString()+")";
            }
            setPartName(partName);
        } catch (CoreException e) {
            e.printStackTrace();
        }

        setSite(site);
        setInput(input);
        TreesTracker.getTracker().addListener(treeStateListener);
        getSite().setSelectionProvider(this);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    protected String node2jsonString(TreeNode n) throws JSONException {
        String result;
        JSONObject json = NodeMapper.toJSON(n);
        json.put("id", n.getId());
        JSONArray jsonChildren = new JSONArray();
        TreeNode[] children = n.getSortedChildren();
        for (int i = 0; i < children.length; i++) {
            jsonChildren.put(children[i].getUUId().toString());
        }
        json.put("children", jsonChildren);

        JSONObject attr = json.getJSONObject("attributes");
        @SuppressWarnings("rawtypes")
        Iterator keys = attr.keys();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            JSONObject a = attr.getJSONObject(key);
            if (n.getQualifiedId().equals(Requirement.getTypeRootQId())) {
                if (a.getString("key").equals("_type")) {
                    a.putOpt("value", null);
                }
            }
            if (a.getString("key").equals(TreeNode.ATTR_DESCRIPTION)) {
                a.putOpt("value", n.getDescription(true));
            }
        }
        result = json.toString(4);
        return result;
    }

    private boolean isWindowsLocalPath(String path) {
        if (path.length() >= 2
                && (path.charAt(0) >= 'a' && path.charAt(0) <= 'z' || path
                        .charAt(0) >= 'A' && path.charAt(0) <= 'Z')
                && (path.charAt(1) == ':' || path.charAt(1) == '|'))
            return true;
        return false;
    }

    private String copyImages(String descr, TreeNode n) {
        Pattern pattern = Pattern
                .compile("<img\\s+[^>]*src=\"([^\\\"]+)\"\\s*[^>]*>");
        Matcher matcher = pattern.matcher(descr);
        while (matcher.find()) {
            String src = matcher.group(1);
            String pathToImage = src;
            pathToImage = pathToImage.replaceAll("\\\\", "/");
            // DELETE "file:/" PREFIXES
            // remote paths cases
            if (pathToImage.startsWith("file://")) {
                // Windows' shares
                if (pathToImage.startsWith("file://///")) {
                    pathToImage = pathToImage.substring(8);
                } else
                // "file://C:/Folder/..."
                if (isWindowsLocalPath(pathToImage.substring(7)) ||
                // "file:///C:/Folder/...", "file:///home/..." - remove
                // "file://"
                        pathToImage.startsWith("file:///")) {
                    pathToImage = pathToImage.substring(7);
                }
                // else "file://compname/share", prefix will be deleted below
            }
            // local paths cases - "file:/C:/Folder/...", "file:/home/..." -
            // remove "file:"
            if (pathToImage.startsWith("file:")) {
                pathToImage = pathToImage.substring(5);
            }
            // delete excess "/" on Windows
            if (pathToImage.startsWith("/")
                    && isWindowsLocalPath(pathToImage.substring(1))) {
                pathToImage = pathToImage.substring(1);
            }
            try {
                // search source file
                File srcFile = new File(pathToImage);
                if (srcFile.isAbsolute()) {
                    if (!srcFile.exists()) { // in URI
                        String absPath = srcFile.getAbsolutePath();
                        absPath = absPath.replaceAll("\\\\", "/");
                        if (isWindowsLocalPath(absPath)) {
                            absPath = "/" + absPath;
                        }
                        if (absPath.startsWith("//")) { // Windows' shares
                            absPath = "///" + absPath;
                        }
                        srcFile = new File(URI.create("file:" + absPath));
                    }
                } else {
                    // suppose, file in URI format
                    String rootPath = n.getTreeDB().getRootNode()
                            .getResourcesUrl().toString();
                    if (rootPath.endsWith("_resources")) { // hack
                        rootPath = rootPath
                                .substring(0, rootPath.length() - 10) + "root";
                    }
                    try {
                        srcFile = new File(URI.create(rootPath + '/'
                                + pathToImage));
                    } catch (IllegalArgumentException e) {
                        // no, file was not URI
                        File rootFile = new File(URI.create(rootPath));
                        // pathToImage = pathToImage.replaceAll("/",
                        // File.separator);
                        srcFile = new File(rootFile + File.separator
                                + pathToImage);
                    }
                }
                if (!srcFile.exists()) {
                    continue;
                }
                // now srcFile is absolute not URI path
                // check destination file
                File dstFile = new File(URI.create(n.getResourcesUrl()
                        .toString()
                        + '/'
                        // short file name in URI (with %20 as ' ' etc.)
                        + (new File(srcFile.toURI().toURL().toString()))
                                .getName()));
                if (srcFile.getAbsolutePath().equals(dstFile.getAbsolutePath())) {
                    continue;
                }
                // copy
                try {
                    FileInputStream in = new FileInputStream(srcFile);
                    n.setResourceContent(dstFile.getName(), in);
                    in.close();
                    descr = descr.replace(src, dstFile.toURI().toURL()
                            .toString());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }
        return descr;
    }

    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            CommonRemoveLocation.RemoveLocation(loc, req, null, true);
            //TODO: save jsData for undo/redo
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }

    @Override
    public void createPartControl(Composite parent) {
        browser = new Browser(parent, RequalityPlugin.BROWSER_TYPE);
        browser.addMouseListener(new MouseListener() {
            @Override
            public void mouseUp(MouseEvent e) {
            }
            
            @Override
            public void mouseDown(MouseEvent e) {
                JSEditor.this.setFocus();
            }
            @Override
            public void mouseDoubleClick(MouseEvent e) {
            }
        });

        browser.addProgressListener(new ProgressListener() {
            
            @Override
            public void completed(ProgressEvent event) {
                setSelection(currentSelection);
            }
            
            @Override
            public void changed(ProgressEvent event) {
            }
        });

        try {
            browser.setUrl(html.toURI().toURL().toString());

            new BrowserFunction(browser, "getNodeByUUID") {
                @Override
                public Object function(Object[] arguments) {
                    UUID nodeUUID = UUID.fromString((String) arguments[0]);
                    TreeNode n = db.getNode(nodeUUID);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };

            new BrowserFunction(browser, "getNodeByQID") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode((String) arguments[0]);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };
            /*
             * Request for the root element of the selected subtree Input
             * parameters: none Return value: String value containing uuid of
             * the subtree root node
             */
            new BrowserFunction(browser, "getStartNode") {
                @Override
                public Object function(Object[] arguments) {
                    return node.getUUId().toString();
                }
            };

            /*
             * Request to change the node ID Input parameters: uuid - (String)
             * uuid of the updated node new_id - (String) new ID to be set
             * Return value: Boolean value: true if operation was successful,
             * false otherwise
             */
            new BrowserFunction(browser, "changeNodeId") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String newId = (String) arguments[1];
                    db.startTransaction("Change Node Id");
                    n.setId(newId);
                    db.commit();
                    return true;
                }
            };

            new BrowserFunction(browser, "changeNodeName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    if (!n.getType().equals("Requirement")) {
                        return false;
                    }
                    String newName = (String) arguments[1];
                    db.startTransaction("Change Node Name");
                    ((Requirement) n).setName(newName);
                    n.saveAttributes();
                    db.commit();
                    return true;
                }
            };

            /*
             * Request to change the value of one of the node attributes Input
             * parameters: uuid - (String) uuid of the updated node attr_name -
             * (String) name of the changed attribute new_val - (String) new
             * attribute value to be set Return value: Boolean value: true if
             * operation was successful, false otherwise
             */
            new BrowserFunction(browser, "changeNodeAttr") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        UUID uuid = UUID.fromString((String) arguments[0]);
                        TreeNode n = db.getNode(uuid);
                        String key = (String) arguments[1];
                        String value = (String) arguments[2];
                        synchronized (JSEditor.this) {
                            db.startTransaction("Change Node Attribute");
                            if (key.equals(TreeNode.ATTR_DESCRIPTION)) {
                                value = copyImages(value, n);
                                value = value.replace(
                                        "\"" + n.getResourcesUrl(),
                                        "\"${node.resURL}");
                            }
                            Attribute attr = new Attribute(
                                    AttributeType.STRING, key, value);
                            n.putAttribute(attr);
                            if (!changedNodes.containsKey(uuid)) {
                                changedNodes.put(uuid, n.toNodeDesc());
                                changedNodesCount.put(uuid, 1);
                            } else {
                                NodeDesc nodeDesc = changedNodes.get(uuid);
                                nodeDesc.setAttribute(attr);
                                changedNodesCount.put(uuid,
                                        changedNodesCount.get(uuid) + 1);
                            }
                            n.saveAttributes();
                            db.commit();
                        }
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                    return true;
                }
            };

            /*
             * Request to change the move a node into different subtree Input
             * parameters: uuid - (String) uuid of the moved node
             * new_parent_uuid - (String) uuid of the new parent node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "changeNodeParent") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[1]));
                    if (!n.getParent().equals(parent)) {
                        db.startTransaction("Move Node");
                        n.setParent(parent);
                        db.commit();
                        return true;
                    }
                    return false;
                }
            };

            /*
             * Request to insert a new node Input parameters: uuid - (String)
             * uuid of the new node parent_uuid - (String) uuid of the parent
             * node Return value: Boolean value: true if operation was
             * successful, false otherwise
             */
            new BrowserFunction(browser, "addNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String id = (String) arguments[1];
                    String type = (String) arguments[2];
                    if (id == null || id.length() == 0) {
                        if (type.equals("Comment")) {
                            id = RequalityCLI.getNextId(parent, type, "Comment ");
                        } else {
                            id = RequalityCLI.getNextId(parent, type, null);
                        }
                    }
                    if (parent.findChild(id) == null) {
                        db.startTransaction("Add Node");
                        TreeNode newNode = parent.createChild(id, type);
                        //TODO: remake to commands
                        if (type.equals("TestPurpose")) {
                            newNode.putAttribute(new Attribute(AttributeType.STRING,
                                    TestPurpose.ATTR_AUTHOR, System.getProperty("user.name", "")));
                            newNode.putAttribute(new Attribute(AttributeType.STRING,
                                    TestPurpose.ATTR_STATUS, "in process"));
                            newNode.saveAttributes();
                        }
                        db.commit();
                        return newNode.getUUId().toString();
                    }
                    return null;
                }
            };

            /*
             * Request to remove a node with all its decendants Input
             * parameters: uuid - (String) uuid of the removed node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "removeNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    db.startTransaction("Remove Node");
                    if (n instanceof Requirement) {
                        deleteLocations((Requirement)n);
                    }
                    n.delete();
                    db.commit();
                    return true;
                }
            };
            new BrowserFunction(browser, "setRequalityFocus") {
                @Override
                public Object function(Object[] arguments) {
                    // workaround (ignore first unieditor selection change)
                    if(currentSelection==null) {
                        currentSelection = targetNode!=null?targetNode:db.getNode(UUID
                                .fromString((String) arguments[0]));
                    } else {
                        currentSelection = db.getNode(UUID
                                .fromString((String) arguments[0]));
                    }
                    if (uniEditorOutlinePage != null) {
                        uniEditorOutlinePage
                                .setSelection(new StructuredSelection(
                                        currentSelection));
                    }
                    // synchronization with Requality Explorer
                    IViewPart view = PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getActivePage()
                            .findView(RequalityCNF.ID);
                    if (view instanceof CommonNavigator) {
                        CommonNavigator navigator = (CommonNavigator) view;
                        if (navigator.isLinkingEnabled()) {
                            navigator.setLinkingEnabled(true);
                        }
                    }
                    fireSelectionChange();
                    return true;
                }
            };

            /**
             * returns absolute URL to project folder
             */
            new BrowserFunction(browser, "getBaseUrl") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        return RequalityPlugin.getTreesTracker().getProject(db)
                                .getLocation().toFile().toURI().toURL()
                                .toString();
                    } catch (MalformedURLException e) {
                        e.printStackTrace();
                    }
                    return null;
                }
            };

            new BrowserFunction(browser, "getReadOnlyMode") {
                @Override
                public Object function(Object[] arguments) {
                    return editMode;
                }
            };

            /**
             * uuid - src node uuid new_parent_uuid - dst node (new parent) uuid
             * deep - optional argument (default deep == true) copy subtree
             * returns absolute URL to project folder
             */
            new BrowserFunction(browser, "copyNode") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        TreeNode srcNode = db.getNode(UUID
                                .fromString((String) arguments[0]));
                        TreeNode newParent = db.getNode(UUID
                                .fromString((String) arguments[1]));
                        Boolean deep = true;
                        if (arguments.length > 2) {
                            deep = (Boolean) arguments[2];
                        }
                        NodeDesc nd = srcNode.toNodeDesc(deep);
                        if (newParent.findChild(nd.getId()) != null) {
                            nd.setId(RequalityCLI.getNextId(newParent, srcNode.getType(), null));
                        }
                        db.startTransaction("Copy Node");
                        TreeNode newNode = nd.createNode(srcNode.getTreeDB(),
                                newParent.getUUId(), deep);
                        db.commit();
                        // FIXME: DataBase affected???
                        copyNodeResources(srcNode, newNode);
                        return newNode.getUUId().toString();
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                    return null;
                }

                private void copyNodeResources(TreeNode srcNode,
                        TreeNode newNode) {
                    IFolder srcFolder = TreesTracker
                            .getResourceByNode(srcNode)
                            .getParent()
                            .getFolder(
                                    Path.fromOSString(srcNode.getId()
                                            + "_resources"));
                    if (!srcFolder.exists()) {
                        return;
                    }
                    IFolder destination = TreesTracker
                            .getResourceByNode(newNode)
                            .getParent()
                            .getFolder(
                                    Path.fromOSString(newNode.getId()
                                            + "_resources"));
                    try {
                        srcFolder.copy(destination.getFullPath(), false, null);
                    } catch (CoreException e) {
                        e.printStackTrace();
                    }

                }
            };
        } catch (IOException e) {
            e.printStackTrace();
        }
        db.addListener(treeChangeListener);
    }

    @Override
    public void setFocus() {
        new UndoRedoActionGroup(getSite(), PlatformUI.getWorkbench()
                .getOperationSupport().getUndoContext(), true)
                .fillActionBars(getEditorSite().getActionBars());
        browser.setFocus();
    }

    public void dispose() {
        db.removeListener(treeChangeListener);
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    public JSEditor() {
        super();
        try {
            html = new File(Activator.getPath("jstools/unieditor.html"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected void refresh(final TreeNode node) {
        // Refresh nothing but root node subtree
        String rootQID = this.node.getQualifiedId();
        String nodeQID = node.getQualifiedId();
        if (!nodeQID.startsWith(rootQID)) {
            return;
        }

        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (browser != null && !browser.isDisposed()) {
                    synchronized (JSEditor.this) {
                        if (changedNodes.containsKey(node.getUUId())) {
                            NodeDesc nd = changedNodes.get(node.getUUId());
                            if (nd.equals(node.toNodeDesc())) {
                                Integer count = changedNodesCount.get(node
                                        .getUUId());
                                if (count > 0) {
                                    changedNodesCount.put(node.getUUId(),
                                            changedNodesCount.get(node
                                                    .getUUId()) - 1);
                                    return;
                                }
                            }
                            changedNodes.remove(node.getUUId());
                            changedNodesCount.remove(node.getUUId());
                        }
                        browser.execute("onChangeNode('" + node.getUUId()
                                + "');");
                    }
                }
            }
        });
    }

    protected void setUniEditorFocus(final UUID uuId) {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (browser != null && !browser.isDisposed()) {
                    browser.execute("onChangeFocus('" + uuId + "');");
                }
            }
        });
    }

    @SuppressWarnings("rawtypes")
    @Override
    public Object getAdapter(Class required) {
        if (IContentOutlinePage.class.equals(required)) {
            if (uniEditorOutlinePage == null) {
                uniEditorOutlinePage = new UniEditorOutlinePage();
                uniEditorOutlinePage
                        .addSelectionChangedListener(outlineSelectionChangeListener);
                uniEditorOutlinePage.setInput(this.node);
            }
            return uniEditorOutlinePage;
        }
        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(new ITabbedPropertySheetPageContributor() {
                @Override
                public String getContributorId() {
                    return "com.unitesk.requality.views.RequalityPropertyContributor";
                }
            });
        }
        return super.getAdapter(required);
    }

    public Browser getBrowser() {
        return browser;
    }

    public void setEditMode(int editMode) {
        this.editMode = editMode;
    }

    private void closeEditor() {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                getEditorSite().getPage()
                        .closeEditor(JSEditor.this, false);
            }
        });
    }

    // //////////////////////////////////////////////////////////////////////////
    //
    // Selection provider
    //
    // //////////////////////////////////////////////////////////////////////////

    private List<ISelectionChangedListener> selectionChangeListeners = new ArrayList<ISelectionChangedListener>();

    @Override
    public void addSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangeListeners.add(listener);
    }

    @Override
    public void removeSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangeListeners.remove(listener);
    }

    @Override
    public ISelection getSelection() {
        if (currentSelection == null) {
            return StructuredSelection.EMPTY;
        }
        return new StructuredSelection(currentSelection);
    }

    @Override
    public void setSelection(ISelection selection) {
    }

    private void fireSelectionChange() {
        ISelection sel = this.getSelection();
        SelectionChangedEvent event = new SelectionChangedEvent(this, sel);
        for (ISelectionChangedListener listener : selectionChangeListeners) {
            listener.selectionChanged(event);
        }
    }
}