/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.documents;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.IDocumentProcessor;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.nodetypes.Document;

// TODO: headless document processing support

public abstract class DocumentProcessor implements IDocumentProcessor {

    protected String encoding = "UTF-8";

    @Override
    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    @Override
    public void process(String docQId, TreeNode rootNode) throws IOException {
        Shell parent = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                .getShell();

        TreeDB db = rootNode.getTreeDB();
        db.startTransaction("Document Processor");
        try {
            Document doc = (Document) db.getNode(docQId);
            String htmlName = doc.getResourceName();
            Reader reader = new InputStreamReader(doc.getResourceContent(htmlName),
                    encoding);

            ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
            Writer writer = new OutputStreamWriter(byteOut, encoding);

            process(docQId, reader, writer, rootNode);
            writer.close();
            reader.close();

            doc.setResourceContent(htmlName, new ByteArrayInputStream(byteOut.toByteArray()));
            db.commit();

            MessageDialog.openInformation(parent, "Document Processor",
                    "Document processing finished");

        } catch (Exception e) {
            db.rollback();
            e.printStackTrace();
            Status status = new Status(IStatus.ERROR, RequalityPlugin.ID, 0,
                    e.getMessage(), null);

            ErrorDialog.openError(parent, "Document Processor Error",
                    "Document Processor Error: " + e.getLocalizedMessage(), status);
        }
    }

    protected abstract void process(String docId, Reader reader, Writer writer,
            TreeNode rootNode) throws IOException;

    @Override
    public boolean configure(Document doc) {
        return true;
    }
}
