/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.handlers.requirement.OpenRequirementHandler;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectReqDialog;
import com.unitesk.requality.nodetypes.Requirement;

public class BaseRequirementPanel {
    private Composite panel;
    private static final String EDIT_BASES = "Base requirements:";
    private static final String BUT_ADD = "Add...";
    private static final String BUT_REMOVE = "Remove...";
    static private String TITLE_ADD_BASE = "Add base requirement";
    static private String TITLE_REMOVE_BASE = "Remove base requirement";

    private TreeViewer basesTree;
    private Button addBaseButton;
    private Button removeBaseButton;

    private Menu basesMenu;
//    static private int i = 0;
    private Requirement requirement;
    private List<String> baseReqIds = new ArrayList<String>();
    private IDoubleClickListener openBaseLink = new IDoubleClickListener() {

        @Override
        public void doubleClick(DoubleClickEvent event) {
            // TODO: move this logic to command handler
            IStructuredSelection selection = (IStructuredSelection) event
                    .getSelection();

//            PropertySheet prop = (PropertySheet) PlatformUI.getWorkbench()
//                    .getActiveWorkbenchWindow().getActivePage().getActivePart();

            RequalityCNF reqExpl = (RequalityCNF) PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage()
                    .findView(RequalityCNF.ID);
            reqExpl.getCommonViewer().setSelection(selection);
            PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                    .getActivePage().activate(reqExpl);
//            IWorkbenchPage page = PlatformUI.getWorkbench()
//                    .getActiveWorkbenchWindow().getActivePage();
//            String viewId = "org.eclipse.ui.views.PropertySheet";
            IHandlerService handlerService = (IHandlerService) PlatformUI
                    .getWorkbench().getActiveWorkbenchWindow().getActivePage()
                    .getActivePart().getSite()
                    .getService(IHandlerService.class);
            try {
                handlerService.executeCommand(
                        "com.unitesk.requality.commands.OpenNode", null);
            } catch (Exception e) {
                e.printStackTrace();
            }
            OpenRequirementHandler.showProperties();
//            if (prop.isPinned()) {
//                try {
//                    page.showView(viewId, String.valueOf("req_prop_" + i),
//                            IWorkbenchPage.VIEW_ACTIVATE);
//                } catch (PartInitException e) {
//                    e.printStackTrace();
//                }
//                i = i + 1;
//            }
        }
    };

    public void setRequirement(Requirement req) {
        requirement = req;
        if (panel.isDisposed()) {
            return;
        }
        baseReqIds = new ArrayList<String>(req.getBaseNodes());
        basesTree.setInput(baseReqIds);
    }

    public BaseRequirementPanel(Composite parent,
            TreeNodePropertiesTab advancedPropertySection,
            TabbedPropertySheetPage propSheet) {
        TabbedPropertySheetWidgetFactory wf = propSheet.getWidgetFactory();

        GridLayout layout = new GridLayout(2, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        
        panel = wf.createComposite(parent, SWT.NONE);
        panel.setLayout(layout);
        
        GridData gridData = new GridData();
        gridData.horizontalSpan = 2;
        wf.createLabel(panel, EDIT_BASES).setLayoutData(gridData);
        
        basesTree = new TreeViewer(panel);
        basesTree.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        basesTree.setContentProvider(baseReqContentProvider);
        basesTree.setLabelProvider(WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider());
        basesTree.addDoubleClickListener(openBaseLink);
        // basesTree.addListener( SWT.KeyDown, addBaseLink );
        // basesTree.addListener( SWT.KeyDown, openBaseLink );
        basesTree.getControl().addListener(SWT.KeyDown, removeBaseLink);

        // Создаем контекстное меню

        basesMenu = new Menu(basesTree.getControl());
        basesTree.getControl().setMenu(basesMenu);
        // basesMenu.addListener( SWT.Show, baseMenuLink );

        // openBaseItem= new MenuItem( basesMenu, SWT.NONE );
        // openBaseItem.setText( TITLE_OPEN_BASE );
        // openBaseItem.addListener( SWT.Selection, openBaseLink );

        // removeBaseItem= new MenuItem( basesMenu, SWT.NONE );
        // removeBaseItem.setText( TITLE_REMOVE_BASE );
        // removeBaseItem.addListener( SWT.Selection, removeBaseLink );

        Composite buttons = wf.createComposite(panel, SWT.NONE);
        FillLayout buttonsLayout = new FillLayout();
        buttonsLayout.spacing = 3;
        buttonsLayout.type = SWT.VERTICAL;
        buttons.setLayout(buttonsLayout);
        buttons.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
        
        // Создаем кнопку "Добавление"
        addBaseButton = wf.createButton(buttons, BUT_ADD, SWT.PUSH);
        addBaseButton.setToolTipText(TITLE_ADD_BASE);

        // Создаем кнопку "Удаление"
        removeBaseButton = wf.createButton(buttons, BUT_REMOVE, SWT.PUSH);
        removeBaseButton.setToolTipText(TITLE_REMOVE_BASE);

        // Реакция на нажатие соответствующей кнопки
        addBaseButton.addListener(SWT.Selection, addBaseLink);
        removeBaseButton.addListener(SWT.Selection, removeBaseLink);

        // Реакция на нажатие клавиши "вниз"
        // addBaseButton.addListener( SWT.KeyDown, buttonLink );
        // removeBaseButton.addListener( SWT.KeyDown, buttonLink );
        initModifyListener();
    }

    protected void updateUI() {
        removeBaseButton.setEnabled(!basesTree.getSelection().isEmpty());
    }

    protected void initModifyListener() {
        updateUI();
        basesTree.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                updateUI();
            }
        });
    }

    public boolean isChanged() {
        return !requirement.getBaseNodes().equals(baseReqIds);
    }

    public List<String> getBaseReqIds() {
        return baseReqIds;
    }

    private Listener addBaseLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.INSERT)
                    || (event.type == SWT.Selection)) {
                SelectNodeDialog selDialog = new SelectReqDialog(
                        panel.getShell(), requirement.getTreeDB());
                if (selDialog.open() == SelectNodeDialog.OK) {
                    Requirement selected = (Requirement) selDialog.getResult();
                    if (selected != null) {
                        if (baseReqIds.contains(selected.getUUId())) {
                            MessageDialog.openInformation(panel.getShell(),
                                    "Duplicate base requirement",
                                    "Base requirement is already in list");
                            return;
                        }

                        baseReqIds.add(selected.getQualifiedId());
                        basesTree.refresh();
                        panel.notifyListeners(SWT.Modify, new Event());
                        requirement.getTreeDB().startTransaction("Add Base Node");
                        requirement.addBaseNode(selected.getQualifiedId());
                        requirement.saveAttributes();
                        requirement.getTreeDB().commit();
                    }
                }
            }
        }
    };

    Listener removeBaseLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {
                if (basesTree.getSelection() instanceof ITreeSelection) {
                    ITreeSelection treeSel = (ITreeSelection) basesTree
                            .getSelection();
                    if (treeSel.getFirstElement() instanceof Requirement) {
                        Requirement selReq = (Requirement) treeSel
                                .getFirstElement();
                        baseReqIds.remove(selReq.getQualifiedId());
                        panel.notifyListeners(SWT.Modify, new Event());
                        basesTree.refresh();
                        basesTree.getControl().setFocus();
                        requirement.getTreeDB().startTransaction("Remove Base Node");
                        requirement.removeBaseNode(selReq.getQualifiedId());
                        requirement.saveAttributes();
                        requirement.getTreeDB().commit();
                    }
                }
            }
        }
    };

    private ITreeContentProvider baseReqContentProvider = new ITreeContentProvider() {

        @Override
        public void inputChanged(Viewer arg0, Object arg1, Object arg2) {

        }

        @Override
        public void dispose() {

        }

        @Override
        public Object[] getElements(Object element) {
            return getChildren(element);
        }

        @Override
        public boolean hasChildren(Object element) {
            return false;
        }

        @Override
        public Object getParent(Object element) {
            return null;
        }

        @SuppressWarnings("unchecked")
        @Override
        public Object[] getChildren(Object parentElement) {
            if (parentElement instanceof List<?>) {
                List<String> listIds = (List<String>) parentElement;
                List<TreeNode> result = new ArrayList<TreeNode>();
                for (String baseId : listIds) {
                    result.add(requirement.getTreeDB().getNode(baseId));
                }

                return result.toArray(new TreeNode[0]);
            }
            return null;
        }
    };

	public Control getControl() {
		return panel;
	}
}
