/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;

public class MainPropertySection extends TreeNodePropertiesTab {

	private AttributesPanel attribPane;
	private Text textId;
	private Text textName;
	ControlDecoration reqIdDecorator;
    private Requirement req;

	public MainPropertySection() {
	}

	@Override
	public void createControls(Composite parent,
			TabbedPropertySheetPage aTabbedPropertySheetPage) {
		super.createControls(parent, aTabbedPropertySheetPage);
		TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
		parent.setLayout(new GridLayout(2, false));
		
		wf.createLabel(parent, "Id:").setLayoutData(
			new GridData(GridData.HORIZONTAL_ALIGN_END)
			);
		textId = wf.createText(parent, "");
		textId.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		reqIdDecorator = createDecorator(textId);
		reqIdDecorator.hide();
		textId.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				if (isReqIdOk(textId.getText())) {
					reqIdDecorator.hide();
				} else {
					if(textId.getText().trim().equals("")) {
						reqIdDecorator.setDescriptionText("Requirement Id should not be empty");
					} else {
					reqIdDecorator.setDescriptionText("Requirement with same Id already exists");
					}
					reqIdDecorator.show();
				}				
			}
			
		});
		textId.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
			    if(req==null) {
			        return;
			    }
				if (req.getId().equals(textId.getText())) {
					return;
				}
				if (!req.getQualifiedId().equals(Requirement.getTypeRootQId())) {
				    String newId = textId.getText().trim();
					if (isReqIdOk(newId)) {
					    req.getTreeDB().startTransaction("Change Requirement Id");
						req.setId(newId.trim());
						req.getTreeDB().commit();
					} else {
						textId.setText(req.getId());
					}
				}
			}
		});
		wf.createLabel(parent, "Name:").setLayoutData(
			new GridData(GridData.HORIZONTAL_ALIGN_END)
			);
		textName = wf.createText(parent, "");
		textName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		textName.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
			    if(req==null) {
			        return;
			    }
				if (req.getName().equals(textName.getText())) {
					return;
				}
                if (textName.getText().length() > 0) {
                    req.getTreeDB().startTransaction("Update Requirement");
                    req.setName(textName.getText());
                    req.saveAttributes();
                    req.getTreeDB().commit();
                } else {
                    textName.setText(req.getId());
                }
			}
		});

		GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalSpan = 2;
		attribPane = new AttributesPanel(parent, wf);
		attribPane.getControl().setLayoutData(gridData);
	}

	private ControlDecoration createDecorator(Text text) {
		ControlDecoration controlDecoration = new ControlDecoration(text,
				SWT.LEFT | SWT.TOP);
		FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
				.getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
		controlDecoration.setImage(fieldDecoration.getImage());
		return controlDecoration;
	}

	protected boolean isReqIdOk(String text) {
	    if( this.req == null || req.getParent() == null ) return false;
		for (TreeNode req : this.req.getParent().getSortedChildren(
				"Requirement")) {
            if(text.trim().length()==0) {
                return false;
            }
			if (req.getId().equals(text.trim()) && !req.getUUId().equals(this.req.getUUId()))
				return false;
		}
		return true;
	}

	@Override
	protected void setTreeNode(TreeNode node) {
	    req = toRequirement(node);
		if (req==null || textId == null || textId.isDisposed()) {
			return;
		}
		if (!req.getQualifiedId().equals(Requirement.getTypeRootQId()))
			textId.setEnabled(true);
		else
			textId.setEnabled(false);
		textId.setText(req.getId());
		textName.setText(req.getName());
		attribPane.setNode(req);
	}
}
