package com.unitesk.requality.merge;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Context {

    private long upperBound;
    private long lowerBound;
    private long width;
    private long label;
    private int id;
    private int sibCount;
    private int position;
    private int depth;
    private int index;
    private Context parentContext;
    private List<Context> childContexts;
    private Node associatedNode;
    private boolean excludeFromSearch = false;
    private static int counter;

    public Context(Node root) {
	associatedNode = root;
	upperBound = Long.MAX_VALUE;
	lowerBound = 0;
	width = upperBound - lowerBound;
	label = makeLabel();
	counter = 1;
	id = counter;
	sibCount = 0;
	position = 1;
	depth = 0;
	index = 1;
	parentContext = null;
	childContexts = new ArrayList<Context>();
    }

    public Context(Node node, Context parentContext) {
	associatedNode = node;
	sibCount = node.getDomNode().getParentNode().getChildNodes()
		.getLength();
	position = getNodePosition();
	width = parentContext.width / sibCount;
	upperBound = parentContext.lowerBound + width * position;
	lowerBound = upperBound - width;
	label = makeLabel();
	counter++;
	id = counter;
	this.parentContext = parentContext;
	depth = parentContext.depth + 1;
	childContexts = new ArrayList<Context>();
	parentContext.addChildContext(this);
	index = getNodeIndex();
    }

    private int getNodePosition() {
	int position = 1;
	for (org.w3c.dom.Node x = associatedNode.getDomNode(); x
		.getPreviousSibling() != null; x = x.getPreviousSibling())
	    position++;
	return position;
    }

    private int getNodeIndex() {
	int index = 1;
	for (int i = 0; i < parentContext.childContexts.indexOf(this)
		&& i < parentContext.childContexts.size(); i++)
	    if (parentContext.childContexts.get(i).associatedNode.getDomNode()
		    .getNodeName()
		    .equals(this.associatedNode.getDomNode().getNodeName()))
		index++;
	return index;
    }

    private long makeLabel() {
	Random rnd = new Random(System.nanoTime());
	long label = lowerBound + width / 2;
	label += (long) (rnd.nextDouble() * width / 4);
	return label;
    }

    private void addChildContext(Context context) {
	if (childContexts.isEmpty())
	    childContexts.add(context);
	else if (childContexts.get(childContexts.size() - 1).label < context.label)
	    childContexts.add(context);
	else {
	    for (int i = 0; i < childContexts.size(); i++) {
		Context prev = childContexts.get(i);
		if (prev.label > context.label)
		    childContexts.add(i, context);
	    }

	}
    }

    public List<Context> getChildContexts() {
	return childContexts;
    }

    public List<Context> getSiblingContexts() {
	if (parentContext == null)
	    return new ArrayList<Context>();
	else
	    return parentContext.childContexts;
    }

    private List<Context> getTextLeaves() {
	List<Context> textLeaves = new ArrayList<Context>();
	if (this.associatedNode.isLeaf() && this.associatedNode.isText()) {
	    textLeaves.add(this);
	    return textLeaves;
	}
	for (Context c1 : this.childContexts) {
	    if (c1.associatedNode.isText())
		textLeaves.add(c1);
	    else if (!c1.associatedNode.isLeaf())
		textLeaves.addAll(c1.getTextLeaves());
	}
	return textLeaves;
    }

    private int contextDistance(Context that) {
	int distance = 0;
	distance += Math.abs(this.id - that.id);
	distance += Math.abs(this.position - that.position);
	distance += Math.abs(this.depth - that.depth);
	distance += Math.abs(this.sibCount - that.sibCount);
	distance += Math.abs(this.index - that.index);
	if (this.parentContext != null && that.parentContext != null)
	    distance += this.parentContext.contextDistance(that.parentContext);
	return distance;
    }

    private double labelCoeff(Context that) {
	double result = 1 + ((double) Math.abs(this.label - that.label))
		* (Math.abs(this.depth - that.depth) + 1)
		/ Math.min(this.width, that.width);
	return result;
    }

    public int contextSimilarity(Context that) {
	List<Context> thisTextLeaves = this.getTextLeaves();
	List<Context> thatTextLeaves = that.getTextLeaves();
	if (!thisTextLeaves.isEmpty() && !thatTextLeaves.isEmpty()) {
	    String thisText = "";
	    for (Context leaf : thisTextLeaves)
		thisText = thisText.concat(" " + leaf.text());
	    String thatText = "";
	    for (Context leaf : thatTextLeaves)
		thatText = thatText.concat(" " + leaf.text());
	    double similarity = Math.max(
		    Utils.nonSymmetricSimilarity(thisText, thatText),
		    Utils.nonSymmetricSimilarity(thatText, thisText));
	    return (int) (similarity * 100);
	} else
	    return 0;
    }

    private String text() {
	return this.associatedNode.getDomNode().getNodeValue();
    }

    public boolean equals(Context that) {
	if (this.label == that.label)
	    return true;
	else
	    return false;
    }

    public int positionDistance(Context that) {
	double distance = this.contextDistance(that) * this.labelCoeff(that);
	return (int) distance;
    }

    public int getPosition() {
	return position;
    }

    public int getDepth() {
	return depth;
    }

    public boolean isWithin(Context context) {
	if (this.lowerBound >= context.lowerBound
		&& this.upperBound <= context.upperBound)
	    return true;
	return false;
    }

    public Context getParentContext() {
	return parentContext;
    }

    public long getLabel() {
	return label;
    }

    public String toString() {
	return associatedNode.toString();
    }

    public int getIndex() {
	return this.index;
    }

    public int getId() {
	return this.id;
    }

    public Node getNode() {
	return this.associatedNode;
    }

    public boolean isExcluded() {
	return this.excludeFromSearch;
    }

    public void excludeFromSearch(boolean exclude) {
	this.excludeFromSearch = exclude;
    }
}
