package com.unitesk.requality.model;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.WorkbenchAdapter;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.Requirement;

final class RequalityWorkbenchAdapter extends WorkbenchAdapter {
    @Override
    public Object getParent(Object o) {
        TreeNode node = (TreeNode) o;
        return node.getParent();
    }

    @Override
    public String getLabel(Object o) {
        if (o instanceof Requirement) {
            Requirement node = (Requirement) o;
            return node.getName();
        }
        if (o instanceof ReportData) {
            ReportData node = (ReportData) o;
            return node.getId();
        }
        if (o instanceof TreeNode) {
            TreeNode node = (TreeNode) o;
            if (node.getQualifiedId().equals("/")) {
                return TreesTracker.getTracker().getProject(node.getTreeDB())
                        .getName();
            }
            return node.toString();
        }
        return super.getLabel(o);
    }

    @Override
    public ImageDescriptor getImageDescriptor(Object object) {
        if (object instanceof Location) {
            Location loc = (Location) object;
            if (loc.getRequirements().length == 0) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImageDescriptor(ISharedImages.IMG_OBJS_ERROR_TSK);
            }
        }
        if (object instanceof TreeNode) {
            TreeNode node = (TreeNode) object;
            if (node.getQualifiedId().equals("/")) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImageDescriptor(IDE.SharedImages.IMG_OBJ_PROJECT);
            }
            ImageDescriptor result = Activator.getDefault().getImageRegistry()
                    .getDescriptor("nodetype." + node.getType());
            if (result == null) {
                return Activator.getDefault().getImageRegistry()
                        .getDescriptor("nodetype.default");
            }
            return result;
        }
        return super.getImageDescriptor(object);
    }

    @Override
    public Object[] getChildren(Object o) {
        TreeNode node = (TreeNode) o;
        return node.getSortedChildren();
    }
}